@@ -1,4 +1,4 @@
-package Lemonldap::Handlers::Generic4a2;
+package Lemonldap::Handlers::Generic4A2;
 use strict;
 use warnings;
 #####  use ######
@@ -6,18 +6,15 @@ use Apache();
 use Apache::URI();
 use Apache::Constants qw(:common :response);
 use Apache::Session::Memorycached;
-#use Apache::ModuleConfig;
+
 use MIME::Base64;
 use LWP::UserAgent;
 use Lemonldap::Config::Parameters;
-#print STDERR "je passe phase 0\n";
+use Apache::Log();
 
-#if(DEBUG) {
-	use Data::Dumper;
-#}
 #### common declaration #######
 our (@ISA, $VERSION, @EXPORTS);
-$VERSION = '0.11';
+$VERSION = '0.14';
 our $VERSION_LEMONLDAP="1.1" ;
 our $VERSION_INTERNAL="0.03-4" ;
 
@@ -51,6 +48,8 @@ our $RECURSIF;
 our $CACHE1_ENTETE;
 our $GENERAL;
 our $ID_HANDLER_IN_PROCESS;
+our $ICS;
+our $ANONYMOUSFUNC;
 our $NOM=  __PACKAGE__ ;
 #############################
 @ISA = qw(LWP::UserAgent );
@@ -74,44 +73,50 @@ sub handler {
 # I must know if the handler is call in the same  virtualhost ou contener
 #
 #
+####
+##  log initialization 
+##
+my $log=$r->log;
+	my $messagelog;
+
 ### I will try  retieve ID_HANDLER from  httpd conf 
 $ID_HANDLER = $r->dir_config('LemonldapHandlerId');
       if  ($ID_HANDLER) {
-print STDERR "$NOM: Phase : handler initialization LOAD ID_HANDLER httpd.conf:$ID_HANDLER : succeded\n" if $DEBUG ; 
-
+     $messagelog = "$NOM Phase : handler initialization LOAD ID_HANDLER httpd.conf:$ID_HANDLER : succeded"; 
   } else { 
 # I don't find anything for this handler in order to make link with XLM conf section
-print STDERR "$NOM: Phase : handler initialization LOAD ID_HANDLER httpd.conf:failed\n" if $DEBUG ; 
+     $messagelog = "$NOM: Phase : handler initialization LOAD ID_HANDLER httpd.conf:failed"; 
  } 
+	$log->debug($messagelog) ;
+
 ############################################
 
 	unless ($ID_HANDLER eq $ID_HANDLER_IN_PROCESS ) {
 ## It doesn t be in the same context ; 
 	    undef $LDAPCONTROL;
             undef $CLIENT;
+            undef $ICS;
 	}
 ## now I save the context of handler
 	$ID_HANDLER_IN_PROCESS =$ID_HANDLER;
 if ($LDAPCONTROL) {  
-print STDERR "$ID_HANDLER: Phase : handler initialization one step beyond\n" if $DEBUG ; 
+$log->debug("$ID_HANDLER: Phase : handler initialization one step beyond") ; 
 }
 unless ($LDAPCONTROL) {
 
-$DEBUG = $r->dir_config('LemonldapDEBUG')  ;
-print STDERR "$ID_HANDLER: Phase : handler initialization DEBUG => $DEBUG\n" if $DEBUG ; 
-print STDERR "$ID_HANDLER: Phase : handler initialization LOAD XML file from  httpd.conf\n" if $DEBUG ; 
+
+$log->debug ("$ID_HANDLER: Phase : handler initialization LOAD XML file from  httpd.conf"); 
 ### I will try  retieve XML  from  httpd conf 
 $FILE = $r->dir_config('LemonldapConfig');
 $GLUE = $r->dir_config('LemonldapConfigIpcKey');
 
-print STDERR "$ID_HANDLER: Phase : handler initialization LOAD XML file $FILE and $GLUE from  httpd.conf\n" if $DEBUG ; 
-
+$log->debug("$ID_HANDLER: Phase : handler initialization LOAD XML file $FILE and $GLUE from  httpd.conf");
 ### retrieve domain xml 
-print STDERR "$ID_HANDLER: Phase : handler initialization LOAD DOMAIN  httpd.conf\n" if $DEBUG ; 
+$log->debug ("$ID_HANDLER: Phase : handler initialization LOAD DOMAIN  httpd.conf"); 
 ### I will try  retieve domain  from  httpd conf 
 $DOMAIN = $r->dir_config('LemonldapDomain');
 ####
-print STDERR "$ID_HANDLER: Phase : handler initialization LOAD ID_HANDLER httpd.conf\n" if $DEBUG ; 
+$log->debug( "$ID_HANDLER: Phase : handler initialization LOAD ID_HANDLER httpd.conf"); 
 ### I will try  retieve ID_HANDLER from  httpd conf 
 #$ID_HANDLER = $r->dir_config('LemonldapHandlerId');
 #      if  ($ID_HANDLER) {
@@ -128,32 +133,33 @@ print STDERR "$ID_HANDLER: Phase : handler initialization LOAD ID_HANDLER httpd.
 #### I have both
 ####  Normaly the cookie is  got from domain 
 ####  I 'll try to load domain ONLY if httpd.conf don't have domain 
-print STDERR "$ID_HANDLER: Phase : handler initialization try to load XML conf\n" if $DEBUG ; 
+$log-> debug("$ID_HANDLER: Phase : handler initialization try to load XML conf"); 
+
 $CONF= Lemonldap::Config::Parameters->new (
                                                 file => $FILE ,
 					      cache => $GLUE );
 	if ($CONF) {
-print STDERR "$ID_HANDLER: Phase : handler initialization LOAD XML conf :succeded \n" if $DEBUG ; 
+$log->debug("$ID_HANDLER: Phase : handler initialization LOAD XML conf :succeded"); 
 } else {
-print STDERR "$ID_HANDLER: Phase : handler initialization LOAD XML conf : failed \n" if $DEBUG ; 
- 
-}
+$log->debug("$ID_HANDLER: Phase : handler initialization LOAD XML conf : failed"); 
+ }
 
 
 ### here conf from XML is ready 
-print STDERR "$ID_HANDLER: Phase : handler initialization LOAD XML conf\n" if $DEBUG ; 
+$log->debug("$ID_HANDLER: Phase : handler initialization LOAD XML conf"); 
 ### I'll search XML section from ID_HANDLER
 
 if ($ID_HANDLER) {
     my $tmpconf;
- print STDERR "domain $DOMAIN\n";
+$log->debug("$ID_HANDLER: domain matched $DOMAIN"); 
+
     if ($DOMAIN) {  
     	$GENERAL = $CONF->getDomain($DOMAIN) ;
 	my $tmpconf = $GENERAL->{handler}->{$ID_HANDLER}; 
 	$COOKIE = $GENERAL->{Cookie};
         $PORTAL=$GENERAL->{Portal};
         $CACHE =$GENERAL->{Session} ;
-        parseConfig($tmpconf);
+       parseConfig($tmpconf);
     		}  else 		{ 
 	$tmpconf= $CONF->{$ID_HANDLER} ;
         parseConfig($tmpconf); 
@@ -199,12 +205,19 @@ $RECURSIF=$_mode  if $_mode;
 my $_proxyext= $r->dir_config('LemonldapProxyExt');
 $PROXYEXT=$_proxyext  if $_proxyext;
 ####################################################
+###   modification for ICS
+###
+####################################################
+my $_ics= $r->dir_config('LemonldapIcs');
+$ICS=$_ics  if $_ics;
+####################################################
+### end   modification for ICS
 
 #
 #
 #
 # Result 
-print STDERR "$ID_HANDLER: Phase : handler initialization VARIABLES
+$log->info("$ID_HANDLER: Phase : handler initialization VARIABLES
 PROXY           => $PROXY
 KEYIPC          => $KEYIPC
 IPCNB           => $IPCNB
@@ -212,19 +225,47 @@ ATTRLDAP        => $ATTRLDAP
 LDAPCONTROL     => $LDAPCONTROL
 DISABLEDCONTROL => $DISABLEDCONTROL
 RECURSIF        => $RECURSIF
-PROXYEXT        => $PROXYEXT 
-STOPCOOKIE      => $STOPCOOKIE\n" if $DEBUG ; 
+PROXYEXT        => $PROXYEXT
+ICS             => $ICS 
+STOPCOOKIE      => $STOPCOOKIE"); 
 }
+## modification for ICS
+if (($ICS) && !($ANONYMOUSFUNC))  {
+my $liste = $ICS ;
+my @lmh= split "," ,$liste;
+$log->debug("$ID_HANDLER: Phase : LOADING ICS TABLE") ; 
+#now I buil the function 
+my $sub = get_match_extension(\@lmh);
+$log->debug ("$ID_HANDLER: Phase : ICS TABLE  LOADED : $sub");
+$ANONYMOUSFUNC='';
+$ANONYMOUSFUNC =eval  $sub ;
+
+}
+
+
+## end modification for ICS
+
+
+
 ##### end of initialization 
-## deleted those line  
 my $uri =$r->uri;
-print STDERR "$ID_HANDLER :uri  requested: $uri\n";
-##### end deleted lines 
+	$log->info ("$ID_HANDLER :uri  requested: $uri");
+	if ($ICS) {
+	my $match = $ANONYMOUSFUNC->($uri);
+# Stop  process  if match 
+if  ($match eq 'OK' ) {
+           
+    $log->info ("$ID_HANDLER :uri ICS matched: $uri");
+           return DECLINED;
+    }
+
+    }
+
 	if($PROXY){
 $UA = __PACKAGE__->new;
 $UA->agent(join "/", __PACKAGE__, $VERSION);
 	
-       print STDERR "$ID_HANDLER:  Build-in proxy actived\n" ;
+      $log->info ("$ID_HANDLER:  Build-in proxy actived") ;
         $r->handler("perl-script");
         $r->push_handlers( PerlHandler => \&proxy_handler );
 	}
@@ -247,11 +288,11 @@ $BASEPRIV=$_basepriv  if $_basepriv;
 my $_portal= $r->dir_config('LemonldapPortal');
 $PORTAL=$_portal  if $_portal;
 ####################################################
-print STDERR "$ID_HANDLER: Phase : handler AUTHORIZATION VARIABLES
+$log->info ( "$ID_HANDLER: Phase : handler AUTHORIZATION VARIABLES
 COOKIE   => $COOKIE
 BASEPUB  => $BASEPUB
 BASEPRIV => $BASEPRIV
-PORTAL   => $PORTAL\n" if $DEBUG ; 
+PORTAL   => $PORTAL");
 
 #}
 #### Read cache info from  XML config 
@@ -261,7 +302,7 @@ PORTAL   => $PORTAL\n" if $DEBUG ;
 my $xmlsession= $CONF->findParagraph('session',$CACHE);
 $SERVERS = $CONF->formateLineHash ($xmlsession->{SessionParams});
 
-print STDERR "$ID_HANDLER: Phase : handler AUTHORIZATION CACHE CONFIG: $SERVERS \n" if $DEBUG ; 
+$log->debug("$ID_HANDLER: Phase : handler AUTHORIZATION CACHE CONFIG: $SERVERS"); 
 }
 #
 #
@@ -269,8 +310,10 @@ print STDERR "$ID_HANDLER: Phase : handler AUTHORIZATION CACHE CONFIG: $SERVERS
 	# AUTHENTICATION
 	# cookie search
 	my $entete2 =$r->headers_in();
-	my %entete =$r->headers_in();
+	#my %entete =$r->headers_in();
 	my $idx =$entete2->{'Cookie'} ;
+	#my %entete =$r->headers_in();
+	#my $idx =$entete{'Cookie'} ;
 	# Load id value from cookie
 #	$idx =~ /$COOKIE=([^; ]+)(;?)/o; 
 # I remove the o option : o parse one time the regexp.
@@ -280,27 +323,28 @@ print STDERR "$ID_HANDLER: Phase : handler AUTHORIZATION CACHE CONFIG: $SERVERS
   #      $id='c167b67d628deb1dcfe09de7aa7f927e';
 	unless ($id) {
 		# No cookie found: redirect to portal
-		print STDERR "$ID_HANDLER : No cookie found for ".$r->uri."\n" if $DEBUG;
+	    $messagelog="$ID_HANDLER : No cookie found for ".$r->uri;
+            $log->info($messagelog);
 		return goPortal($r,'c');
 	}
-	print STDERR "$ID_HANDLER: id session : $id<--->$idx\n" if $DEBUG;
+$log->info("$ID_HANDLER: id session : $id<--->$idx");
 	# SESSIONS CACHE 
 
 #cache  level 1 test 
 	my $ligne_h;
  unless ($id eq $CLIENT) {
             # Level 2 test by IPC 
-      print STDERR "$ID_HANDLER: No match in cache level 1 for $id\n" if $DEBUG;
+     $log->info ("$ID_HANDLER: No match in cache level 1 for $id");
               if ($IPCNB)  {  ####  We want use IPC                 
-                       print STDERR "$ID_HANDLER :  search in cache level 2 for $id\n" if $DEBUG;
+		  $log->info("$ID_HANDLER :  search in cache level 2 for $id");
                 tie %STACK ,'IPC::Shareable' , $KEYIPC, 
                             {create => 1 , mode => 0666};   
                  $ligne_h = $STACK{$id} ;      
                     if  ($ligne_h) {  ## match in ipc 
-                       print STDERR "$ID_HANDLER :  match in cache level 2 for $id\n" if $DEBUG;
+			$log->info  ("$ID_HANDLER :  match in cache level 2 for $id");
                             expire_session($id) ;# put on the top of stack    
                                    } else  { 
-      print STDERR "$ID_HANDLER: No match in cache level 2 for $id\n" if $DEBUG;
+				       $log->info("$ID_HANDLER: No match in cache level 2 for $id");
 }
 
 
@@ -308,7 +352,7 @@ print STDERR "$ID_HANDLER: Phase : handler AUTHORIZATION CACHE CONFIG: $SERVERS
 	                   }  ####  We want use IPC 
 
             unless ($ligne_h) { # no match in cache level 1 and 2 
-		print STDERR "$ID_HANDLER :  Search  in cache level 3 for $id\n" if $DEBUG;
+	       $log->info ("$ID_HANDLER :  Search  in cache level 3 for $id");;
 ###### 
 ######
 ######   search in backend cache 
@@ -319,12 +363,12 @@ print STDERR "$ID_HANDLER: Phase : handler AUTHORIZATION CACHE CONFIG: $SERVERS
                          ##  tree causes : Too many connection are served.              
                          ##                the server of session was restarted                
                          ##                It's time out                 
-           print STDERR "$ID_HANDLER: ERROR OF LOCKING  ON :$id\n"  if $DEBUG; 
+           $log->warning("$ID_HANDLER: ERROR OF LOCKING  ON :$id"); 
 # I say it's time out 
 	return goPortal($r,'t');
                        }
 #here  we are retrieve session        
-           print STDERR "$ID_HANDLER: SESSION FIND FOR:$id\n"  if $DEBUG; 
+           $log->info ("$ID_HANDLER: SESSION FIND FOR:$id"); 
 #now we will look at authorization  and build an header and stock it for the next access
 #FIRST : authorization 
 my $uid = $session{uid};
@@ -351,15 +395,17 @@ my $complement;
 #### begin:  here for he compatibility  with older lemonldap
             $complement=~  s/#.*//;
 ###  end  :  here for he compatibility  with older lemonldap
-         
-                          } 
+
+
+
+                                     } 
 ### end of control                                      
      if ($etat) { 
-	 print STDERR "$ID_HANDLER: controle: $dn $uri :ACCEPTED \n" if $DEBUG;
+	 $log->notice("$ID_HANDLER: controle: $dn $uri :ACCEPTED");
       }   else {
 #	  save_session ($id,'BIDON')  ;
 #print STDERR "$ID_HANDLER: $id saving in cache level 2\n" if $DEBUG;
-         print STDERR "$ID_HANDLER: controle: $dn $uri :DENIED \n" if $DEBUG;
+	  $log->notice("$ID_HANDLER: controle: $dn $uri :DENIED");
       }
 
 untie %session;
@@ -374,10 +420,22 @@ untie %session;
 
 
 ############################################
+##  new feature , in the new release all header is saving in session
+##  info session MUST BE BEGAN with uid..
+if ($complement =~ /^uid/)  {
+$ligne_h = $complement;
+
+
+}  else 
+{
+
 $ligne_h = $dn;
 if (defined($complement)) {
 $ligne_h.=":$complement";
 } 
+}
+
+
 	    } ### end of search in cache 1 ,2 and 3
 
    } ##  end of cache level 1 
@@ -386,7 +444,7 @@ $ligne_h.=":$complement";
 #cache  level 1 test 
 # unless($id eq $CLIENT)
       $ligne_h=$CACHE1_ENTETE; 
-      print STDERR "$ID_HANDLER: match in cache level 1 for $id\n" if $DEBUG;
+      $log->info("$ID_HANDLER: match in cache level 1 for $id");
 
   }
 
@@ -401,16 +459,20 @@ $ligne_h.=":$complement";
 #
 $CLIENT=$id;
 $CACHE1_ENTETE=$ligne_h; 
-print STDERR "$ID_HANDLER: $id saving in cache level 1\n" if $DEBUG;
+$log->info ("$ID_HANDLER: $id saving in cache level 1");
 
       if (($IPCNB) && (defined (%STACK))) { #we want cache IPC level 2
 	  save_session ($id,$ligne_h)  ;
-print STDERR "$ID_HANDLER: $id saving in cache level 2\n" if $DEBUG;
+$log->info( "$ID_HANDLER: $id saving in cache level 2");
           untie %STACK ;
       }
+     ###  add user in access log
+      my ($user) = $ligne_h=~ /(^u.+?),/;
+$r->user($user) if $user ;
+
 my $hcode =encode_base64($ligne_h,''); 
-print STDERR "$ID_HANDLER: header before encoding: $ligne_h\n" if ($DEBUG);
-print STDERR "$ID_HANDLER: header after encoding: $hcode\n" if $DEBUG;
+$log->info ("$ID_HANDLER: header before encoding: $ligne_h");
+$log->info ("$ID_HANDLER: header after encoding: $hcode");
 ###############  We can insert the header #####################
 my $entete_spec = "Basic ";
 $entete_spec.=$hcode."\n";
@@ -422,9 +484,13 @@ $r->header_in('Authorization'=> $entete_spec);
 	    $r->headers_in->do(sub {
 		(my $cle ,my $valeur) = @_;
 		if ($valeur=~ /$COOKIE/) {
-		    my $tmp =~ /$COOKIE=.+b/;
-		    $_[1]=~ s/$tmp//;
-		    print STDERR "$ID_HANDLER: STOPCOOKIE done\n" if ($DEBUG);
+
+		    $_[1]=~ s/$COOKIE=(.+?)\b//;
+		    $_[1]=~ s/;\s+;/;/;
+		    $_[1]=~ s/^\s?;//;
+		    $_[1]=~ s/^\s+;//;
+		   
+		   $log->debug  ("$ID_HANDLER: STOPCOOKIE done");
 		}
 		1;
 	    });
@@ -446,7 +512,10 @@ sub proxy_handler {
         # Transformation: GET /index.html becomes http://servername/index.html
         # $url contains the real value (hided server)
         # $url_init contains the asked value
-    my $url =$r->uri;
+ #add this in order to log
+my $log=$r->log;   
+ my $url =$r->uri;
+   
     $url .="?".$r->args if ($r->args);
     my %entete = $r->headers_in();
     my $url_init= $BASEPUB.$url;
@@ -455,8 +524,9 @@ sub proxy_handler {
 #    $uuu=~ s/handler/ressource/;
 #################################
     $url = $BASEPRIV.$uuu;
- print STDERR "$ID_HANDLER: URLPRIV ACTIVED: $url  
-                     URLPUB REQUESTED : $url_init\n" if ($DEBUG);       
+    $log->info("$ID_HANDLER: URLPRIV ACTIVED: $url  
+                     URLPUB REQUESTED : $url_init");
+       
     my $request = HTTP::Request->new($r->method, $url);
     $r->headers_in->do(sub {
 	$request->header(@_);
@@ -481,9 +551,12 @@ if ($request->header('Host')){
 if ($request->header('Connection')){
       $request->header('Connection' => 'close');
 }
-    print STDERR  "$ID_HANDLER: request ".$request->as_string()."\n" if($DEBUG);
-   if ($RECURSIF) { 
-       print STDERR  "$ID_HANDLER: RECURSIF LWP DESACTIVED\n" if ($DEBUG);
+
+  
+my $messagelog ="$ID_HANDLER: request ".$request->as_string() ;
+ $log->debug($messagelog);  
+ if ($RECURSIF) { 
+     $log->debug  ("$ID_HANDLER: RECURSIF LWP DESACTIVED");
     my @tt= ('HEAD');
     $UA->requests_redirectable(\@tt);
 }
@@ -491,15 +564,16 @@ if ($request->header('Connection')){
         # LWP proxy
 # I 'll forward  on an  external proxy 
 if ($PROXYEXT) {
-print STDERR  "$ID_HANDLER:OUTPUT PROXY:$PROXYEXT\n" if ($DEBUG);
+    $log->debug("$ID_HANDLER:OUTPUT PROXY:$PROXYEXT");
 
     $UA->proxy(http  => $PROXYEXT);
 } 
     my $response = $UA->request($request);
-    my $type =$response->header('Content-type');
 ### begin: somes bad requests have bad header . 
+ $messagelog= $response->as_string();
+    $log->debug($messagelog);
     my $content = $response->header('Content-type');
-    $content=~ s/,/;/g ;
+     $content=~ s/,/;/g ;
 ### end: somes bad requests have bad header . 
     $r->content_type($content);
 ### begin: I correct on the fly some incomming header like mod_proxy does
@@ -509,29 +583,24 @@ if ($response->header('Location')) {
 $response->header('Location' => $h);
 }
 ### end: I correct on the fly some incomming header like mod_proxy does
-     $r->status($response->code);
+
+    $r->status($response->code);
     $r->status_line(join " ", $response->code, $response->message);
     $response->scan(sub {
 	$r->headers_out->add(@_);
     });
-if ($r->header_out('Location')) {
-  my $h =$r->header_out('Location');
-   $h=~ s/$BASEPRIV/$BASEPUB/ ;
-$r->header_out('Location' => $h);
-}
-
-
 
     if ($r->header_only) {
 	$r->send_http_header();
 	return OK;
     }
 
-    my $content = \$response->content;
+     $content = \$response->content;
+    
     $r->content_type('text/html') unless $$content;
     $r->send_http_header;
     $r->print($$content || $response->error_as_HTML);
-    print STDERR "$ID_HANDLER: response sent\n"  if ($DEBUG);
+    $log->notice ("$ID_HANDLER: $url response sent");
     return OK;
 
 
@@ -539,14 +608,16 @@ $r->header_out('Location' => $h);
 
 }
 sub goPortal {
-	my  $r = shift;
-        my $op= shift;
+       my  $r = shift;
+       my $log=$r->log;
+       my $op= shift;
+
         my $urlc_init = $BASEPUB.$r->uri;
         $urlc_init.="?".$r->args if $r->args; 
-    my  $urlc_initenc = encode_base64($urlc_init,"");
-        print STDERR "GERMANEX $urlc_init\n" ;    
-   	$r->header_out(location =>$PORTAL."?op=$op&url=$urlc_initenc");
-	print STDERR "$ID_HANDLER : Redirect to portal (url was ".$urlc_init.")\n" if($DEBUG);
+       my  $urlc_initenc = encode_base64($urlc_init,"");
+       	$r->header_out(location =>$PORTAL."?op=$op&url=$urlc_initenc");
+        my $messagelog =  "$ID_HANDLER : Redirect to portal (url was ".$urlc_init.")";
+$log->info($messagelog); 
 	return REDIRECT;
         exit;
 }
@@ -555,51 +626,51 @@ sub   expire_session {
 my $id = shift;
 tied(%STACK)->shlock ;
 my $tmpvar = $STACK{'QUEUE'};
-#my $ligne= Dumper ($tmpvar);
-#print STDERR "eZZZ : $ligne \n";
 my @tmp ;
 if ($tmpvar) { 
 @tmp= split /#/,$tmpvar ;
 }  
-#$ligne= Dumper (@tmp);
-#print STDERR "eXXX : $ligne \n";
+
 my @stack;
 @stack= grep ($id ne $_ ,@tmp);
 unshift @stack, $id;
-#### for avoid stack overflow 
+
+my $config = \@stack;
+my $warning = $#stack ;
+my $borne = $IPCNB + 4 ; #I keep a small gap
 my %hashs;
+if ($warning >  $borne ) {
+
+
+#### for avoid stack overflow 
+
 foreach (@stack) {
 $hashs{$_}=1;
 
 }
+}
+####
 
-my $config = \@stack;
-#my $configsx=Dumper ($config);
-#print STDERR "final = $configsx\n";
   if ($#stack > $IPCNB ) {
   my $to_delete = pop @stack ;
-  print STDERR "sup  $to_delete\n";
+#  print STDERR "sup  $to_delete\n";
   delete $STACK{$to_delete}; 
    }
 #### for avoid stack overflow 
+if ($warning >  $borne ) {
+
 foreach (keys %STACK)  {
 next if /QUEUE/ ;
 delete $STACK{$_} unless $hashs{$_} ;
  }
+}
 
-
-#$Data::Dumper::Purity=1;
-#$Data::Dumper::Terse=1;
 my $buffer;
 foreach (@stack){
     $buffer.="$_"."#";
 }
 $buffer=~ s/#$//; 
 $config = \@stack;
-#$configsx=Dumper ($config);
-#print STDERR "essss : $configsx \n";
-#my $configs=Dumper ($buffer);
-#print STDERR "errr : $configs \n";
 
 $STACK{'QUEUE'} = $buffer;
 tied(%STACK)->shunlock ;
@@ -611,6 +682,17 @@ tied(%STACK)->shlock ;
 $STACK{$id} = $trace;
 tied(%STACK)->shunlock ;
 }
+ sub get_match_extension   {
+    my $tablemh= shift;
+my $code = "sub {local \$_ = shift;\n"; 
+foreach (@$tablemh) {
+$code .= "return \"OK\"  if /\\.$_\$/i;\n";  
+}
+$code.= "1;}\n";
+return $code;
+}
+
+
 sub parseConfig {
     my $tmp =shift;
     $PROXY= $tmp->{Enabledproxy};
@@ -624,6 +706,12 @@ sub parseConfig {
     $STOPCOOKIE= $tmp->{StopCookie};
     $PROXYEXT = $tmp->{ProxyExt};
     $RECURSIF= $tmp->{Recursive};
+    $PORTAL= $tmp->{Portal} if $tmp->{Portal} ;
+
+## ICS  i must  collect info 
+    $ICS=$tmp->{Ics} ;   
+
+
         }
 1;
 __END__
@@ -631,30 +719,28 @@ __END__
 
 =head1 NAME
 
-Lemonldap::Handlers::Generic4a2 - Handler Apache2 for  Lemonldap sso system
+    Lemonldap::Handlers::Generic - Perl extension for Lemonldap sso system
+
+    Lemonldap::Handlers::Generic4a2 - Handler for Apache2 Lemonldap SSO
+    system
+    
+    Lemonldap::Handlers::GenericSSL - Handler full SSL for Lemonldap sso system
+
+    Lemonldap::Handlers::Generic4a2 - Handler full SSL  Apache2 Lemonldap SSO
+
+    Lemonldap::Handlers::GenericMH1 - Handler with multihoming facilities
+
+
+
 
 =head1 SYNOPSIS
 
 In httpd.conf 
 
- <VirtualHost *:80>
- PerlSetVar LemonldapEnabledproxy 1
- ServerName serverpub.foo.bar:80
- DocumentRoot /usr/local/apache2/htdocs
- PerlInitHandler Lemonldap::Handlers::Generic4a2
- #ProxyPass / http://serverpriv.foo.bar/
- #ProxyPassReverse / http://serverpriv.foo.bar/
- PerlSetVar LemonldapConfig /usr/local/apache2/conf/application_new.xml
- PerlSetVar LemonldapConfigIpcKey CONF
- PerlSetVar LemonldapDEBUG 1
- PerlSetVar LemonldapDomain foo.bar
- PerlSetVar LemonldapHandlerID handler2
- PerlSetVar LemonldapBasePub http://serverpub.foo.bar
- PerlSetVar LemonldapBasePriv http://serverpriv.foo.bar
- PerlSetVar LemonldapCodeAppli APPLI
- PerlSetVar LemonldapAttrLdap profilapplicatif
- </virtualhost>
-
+<location mylocation>  
+ Lemonldap::Handlers::Generic;
+ 
+</location>
 
 =head1 DESCRIPTION
 
@@ -772,14 +858,13 @@ MacEachern - O'REILLY
 
 =item Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
-=item  Portage under Apache2 is made with help of : Ali Pouya and Shervin
-     Ahmadi (MINEFI/DGI)
-     
+=item Isabelle Serre, E<lt>isabelle.serre@justice.gouv.frE<gt>
+
 =back
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2004 by Eric German E<amp> Xavier Guimard
+Copyright (C) 2004 by Eric German E<amp> Xavier Guimard E<amp> Isabelle Serre
 
 Lemonldap originaly written by Eric german who decided to publish him in 2003
 under the terms of the GNU General Public License version 2.
@@ -795,6 +880,8 @@ under the terms of the GNU General Public License version 2.
 =item Portions are copyrighted by Doug MacEachern and Lincoln Stein.
 This library is under the GNU General Public License, Version 2.
 
+=item Portage under Apache2 is made with help of : Ali Pouya and 
+Shervin Ahmadi (MINEFI/DGI) 
 
 =back
 
